@@ -17,15 +17,17 @@ I'll leave this for a later version @@
 */
 
 #include <stdio.h>
-#include <config.h>
 #include <unistd.h>
-#include <fcntl.h>
+#include <config.h>
 #include <gtk/gtk.h>
+#include <fcntl.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/ioctl.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
+#include <net/if.h>
 #include <arpa/inet.h>
 #include <netinet/tcp.h>
 #include <netinet/ip.h>
@@ -43,6 +45,7 @@ I'll leave this for a later version @@
 #include "smashy.h"
 #include "gtk_misc.h"
 #include "scrupdate.h"
+#include "guidefs.h"
 
 /* GLOBALS */
 
@@ -58,10 +61,18 @@ GtkWidget *mainwindow;		/* Main window */
 GtkWidget *protowindow=NULL;	/* Protocols window */
 GtkWidget *protoconfigwindow=NULL;	/* Protocol config window */
 GtkWidget *prototree=NULL;	/* Protocol tree */
+GtkText *savewatchwidget;
 GtkWidget *logtext;		/* Log window */
 GtkWidget *menubar;		/* Pulldown menu bar */
-GtkAcceleratorTable *accel;	/* The accelerator keys */
+#ifdef GTK_HAVE_FEATURES_1_1_0
+ GtkAccelGroup *accel;
+#else
+ GtkAcceleratorTable *accel;	/* The accelerator keys */
+#endif
+GList *interfaces = NULL;
+GtkWidget *cb = NULL;		/* Combo box */
 int protocoltag=0;
+static struct ifreq ifr[16];
 
 extern t_arp_rec *first_arp_rec;
 extern ether_protocols *first_ether_protocol;
@@ -73,7 +84,7 @@ extern int logfd;
 extern int started, selected_row;
 extern t_arp_rec *selected_ar;
 extern t_karpski_stats karpski_stats;
-extern char *device;
+extern char device[];
 extern int all_conns_selected;
 extern int needssave;
 extern pthread_mutex_t ArpRecMutex;
@@ -93,7 +104,7 @@ switchlog(GtkWidget *widget)
    {
       logwindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
       gtk_widget_set_name(logwindow, "log");
-      gtk_widget_set_usize(logwindow, 300,280);
+      gtk_widget_set_usize(logwindow, LOG_WIDTH, LOG_HEIGHT);
       
       gtk_widget_set_uposition(logwindow, 1024-300-5, 0);
 
@@ -201,9 +212,9 @@ switchprotocols(GtkWidget *widget)
    {
       protowindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
       gtk_widget_set_name(protowindow, "Protocols");
-      gtk_widget_set_usize(protowindow, 260,400);
+      gtk_widget_set_usize(protowindow, PROTO_WIDTH, PROTO_HEIGHT);
       
-      gtk_widget_set_uposition(protowindow, 1280-572-5, 0);
+/*      gtk_widget_set_uposition(protowindow, 1280-572-5, 0); */
 
       gtk_window_set_policy (GTK_WINDOW(protowindow), TRUE, TRUE, FALSE);
          
@@ -273,10 +284,8 @@ switchstats(GtkWidget *widget)
    {
       statswindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
       gtk_widget_set_name(statswindow, "Statistics");
-/*      gtk_widget_set_usize(statswindow, 300,280);
+/*      gtk_widget_set_usize(statswindow, STATS_WIDTH, STATS_HEIGHT);  */
       
-      gtk_widget_set_uposition(statswindow, 1024-300-5, 0); */
-
       gtk_window_set_policy (GTK_WINDOW(statswindow), TRUE, TRUE, FALSE);
          
       gtk_signal_connect (GTK_OBJECT (statswindow), "destroy",
@@ -608,6 +617,7 @@ void select_clist(GtkWidget *widget, gint row, gint column, GdkEventButton *beve
    char *machine_addr;
    char maddr2[64];
    
+   
    selected_row = row;
    gtk_clist_get_text(GTK_CLIST(widget), row, 1, &hw_addr);
    gtk_clist_get_text(GTK_CLIST(widget), row, 2, &machine_addr);
@@ -631,11 +641,74 @@ void select_clist(GtkWidget *widget, gint row, gint column, GdkEventButton *beve
    gtk_clist_clear (GTK_CLIST(connectlist));
 
    if (selected_ar)
-      display_connections(selected_ar->first_connect, 0);
+      display_connections(selected_ar->machine_addr, selected_ar->first_connect, 0);
    else
       fprintf(stderr,"Can't find hw_addr %s!!\n", hw_addr);
 }
 
+void save_watch_ok(GtkWidget *w, GtkFileSelection *fs)
+{
+   char fn[512];
+   char c;
+   long watchlen = 0;
+   long i=0;
+   int fd;
+
+   strcpy(fn, gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+   gtk_widget_destroy(GTK_WIDGET(fs));
+   
+   if ((fd = open(fn, O_WRONLY | O_CREAT | O_TRUNC))<0)
+   {
+      perror("Can't open file for writing: ");
+      return;
+   }
+
+   watchlen = gtk_text_get_length(savewatchwidget);
+   
+/*   printf("Writing %ld byte file %s.\n", watchlen, fn); */
+   
+   for (i=0; i<watchlen; i++)
+   {
+      c = GTK_TEXT_INDEX(savewatchwidget, i);
+      if (write(fd, &c, 1)<0)
+      {
+         perror("write watch data");
+         exit(1);
+      }
+   }
+   close(fd);
+
+}
+
+void save_watch(GtkWidget *text)
+{
+   static GtkWidget *window = NULL;
+   
+   
+   if ((!text) || (!GTK_IS_TEXT(text)))
+      return;
+      
+   savewatchwidget = GTK_TEXT(text);
+      
+   if (!window)
+   {
+      window = gtk_file_selection_new("Save watch dialog");
+      gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
+      gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &window);      
+      gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(window)->ok_button),
+                         "clicked", GTK_SIGNAL_FUNC(save_watch_ok),
+                         window);
+      gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(window)->cancel_button),
+                               "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
+                               GTK_OBJECT(window));
+   }
+   if (!GTK_WIDGET_VISIBLE(window))
+      gtk_widget_show(window);
+   else
+      gtk_widget_destroy(window);
+
+}
+
 void switch_watch()
 {
    char *watching;
@@ -680,13 +753,14 @@ void switch_watch()
    {
       gtk_widget_destroy(this_connect->watchwindow);
       this_connect->watchwindow = NULL;
-      gtk_clist_set_text(GTK_CLIST(connectlist), connect_row, 0, "No");
+      this_connect->need_update = 1;
+/*      gtk_clist_set_text(GTK_CLIST(connectlist), connect_row, 0, "No"); */
    }
    else
    {
       this_connect->watchwindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_widget_set_name(this_connect->watchwindow, "Connections");
-      gtk_widget_set_usize(this_connect->watchwindow, 380,280);
+      gtk_widget_set_name(this_connect->watchwindow, "Watchwindoze");
+      gtk_widget_set_usize(this_connect->watchwindow, WATCH_WIDTH, WATCH_HEIGHT);
       
       gtk_widget_set_uposition(this_connect->watchwindow, 619, 300);
 
@@ -732,11 +806,18 @@ void switch_watch()
       separator = gtk_hseparator_new ();
       gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
       gtk_widget_show (separator);
-      box2 = gtk_vbox_new (FALSE, 10);
+      box2 = gtk_hbox_new (FALSE, 10);
       gtk_container_border_width (GTK_CONTAINER (box2), 10);
       gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
       gtk_widget_show (box2);
 
+      button = gtk_button_new_with_label ("save");
+      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+                                GTK_SIGNAL_FUNC(save_watch),
+                                (GtkObject *)this_connect->watchtextbox);
+      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
+      gtk_widget_show (button);
+      
       button = gtk_button_new_with_label ("close");
       gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
       				GTK_SIGNAL_FUNC(switch_watch),
@@ -744,11 +825,13 @@ void switch_watch()
       gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
       gtk_widget_show (button);
       gtk_widget_show (this_connect->watchwindow);
-      gtk_clist_set_text(GTK_CLIST(connectlist), connect_row, 0, "Yes");
+/*      gtk_clist_set_text(GTK_CLIST(connectlist), connect_row, 0, "Yes"); */
+      this_connect->need_update = 1;
    }
 }
 
 
+
 void close_watchall(t_arp_rec *selected_ar)
 {
    if (selected_ar->row<0)
@@ -787,7 +870,7 @@ void switchwatchall()
       selected_ar->watchwindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
       sprintf(strbuf, "Watch %s", selected_ar->machine_addr);
       gtk_widget_set_name(selected_ar->watchwindow, strbuf);
-      gtk_widget_set_usize(selected_ar->watchwindow, 380,280);
+      gtk_widget_set_usize(selected_ar->watchwindow, WATCH_WIDTH, WATCH_HEIGHT);
       gtk_widget_set_uposition(selected_ar->watchwindow, 619, 300);
       gtk_window_set_policy (GTK_WINDOW(selected_ar->watchwindow), TRUE, TRUE, FALSE);
          
@@ -831,11 +914,18 @@ void switchwatchall()
       separator = gtk_hseparator_new ();
       gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
       gtk_widget_show (separator);
-      box2 = gtk_vbox_new (FALSE, 10);
+      box2 = gtk_hbox_new (FALSE, 10);
       gtk_container_border_width (GTK_CONTAINER (box2), 10);
       gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
       gtk_widget_show (box2);
 
+      button = gtk_button_new_with_label ("save");
+      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+                                GTK_SIGNAL_FUNC(save_watch),
+                                (GtkObject *)selected_ar->watchtextbox);
+      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
+      gtk_widget_show (button);
+      
       button = gtk_button_new_with_label ("close");
       gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                                 GTK_SIGNAL_FUNC(close_watchall),
@@ -892,6 +982,7 @@ void getquotstr(char *line, int strnum, char buf[])
 void clearlist()
 {
    t_arp_rec *t_arp, *next_arp;
+
    for (t_arp = first_arp_rec; t_arp; t_arp = next_arp)
    {
       next_arp = t_arp->next;
@@ -899,12 +990,14 @@ void clearlist()
          gtk_widget_destroy(t_arp->watchtextbox);
       if (t_arp->watchwindow)
          gtk_widget_destroy(t_arp->watchwindow);
-      /* Free conections, kill their watches @@@ */
+      clear_and_free_connections(t_arp->first_connect);
       free(t_arp);
    }
    
    first_arp_rec = NULL;
    gtk_clist_clear(GTK_CLIST(main_clist));
+   if (connectlist)
+      gtk_clist_clear(GTK_CLIST(connectlist));
    stablenet = 0;
    bzero(&karpski_stats, sizeof(karpski_stats));
 }
@@ -1072,6 +1165,122 @@ void selection_changed(GtkTree *tree)
 
 }
 
+void change_interface(GtkWidget *widget, GtkWidget *window)
+{
+   gchar *text;
+   
+   text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cb)->entry));
+
+   if (started)
+      printf("Stop capturing first, then select the interface\n");
+   else
+   {
+      strncpy(device, text, IFNAMSIZ);
+
+      if (window)
+      {
+         gtk_widget_destroy(window);
+         window = NULL;
+      }
+      
+      printf("Now capturing from %s\n", device);
+   }
+}
+
+void interfacesel()
+{
+   static GtkWidget *window = NULL;
+   GtkWidget *box1, *box2;
+   
+   struct ifconf ifc;
+   int fd;
+   int x;
+   int num;
+   GtkWidget *button, *separator;
+   char dev[IFNAMSIZ+1];
+   
+   strncpy(dev, device, IFNAMSIZ);
+   
+   if (!window)
+   {
+      if ((fd = socket(AF_INET, SOCK_DGRAM, 0))<0)
+      {
+         fprintf(stderr, "Socket failed in interfacesel\n");
+         exit(1);
+      }
+      
+      ifc.ifc_len = sizeof(ifr);
+      ifc.ifc_buf = (caddr_t)ifr;
+      
+      bzero(&ifr, sizeof(ifr));
+      
+      if (ioctl(fd, SIOCGIFCONF, &ifc) <0 || (ifc.ifc_len < sizeof(struct ifreq)))
+      {
+         fprintf(stderr, "ioctl() SIOCGIFCONF failed.\n");
+         close(fd);
+         exit(1);
+      }
+      
+      close(fd);
+      num = (ifc.ifc_len / sizeof(struct ifreq));
+
+      if (!interfaces)
+      {
+         for (x=0; x<num; x++)
+         {
+            interfaces = g_list_append(interfaces, ifr[x].ifr_name);
+         }
+      }
+      
+      window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+      
+      gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &window);
+      gtk_window_set_title(GTK_WINDOW(window), "Interfaces");
+      gtk_container_border_width(GTK_CONTAINER(window), 0);
+      
+      box1 = gtk_vbox_new(FALSE, 0);
+      gtk_container_add(GTK_CONTAINER(window), box1);
+      gtk_widget_show(box1);
+      
+      box2 = gtk_vbox_new (FALSE, 10);
+      gtk_container_border_width (GTK_CONTAINER (box2), 10);
+      gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
+      gtk_widget_show (box2);
+
+      if ((cb = gtk_combo_new ()) == NULL)
+      {
+         perror("gtk_combo_new()");
+         exit(1);
+      }
+      gtk_combo_set_popdown_strings (GTK_COMBO (cb), interfaces);
+      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO(cb)->entry), device);
+      gtk_editable_select_region (GTK_EDITABLE (GTK_COMBO(cb)->entry), 0, -1);
+      gtk_box_pack_start (GTK_BOX (box2), cb, TRUE, TRUE, 0);
+      gtk_widget_show (cb);
+
+      separator = gtk_hseparator_new ();
+      gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
+      gtk_widget_show (separator);
+                  
+      box2 = gtk_vbox_new (FALSE, 10);
+      gtk_container_border_width (GTK_CONTAINER (box2), 10);
+      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
+      gtk_widget_show (box2);
+                                          
+      button = gtk_button_new_with_label ("close");
+      gtk_signal_connect_object (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC(change_interface), GTK_OBJECT (window));
+      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
+      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+      gtk_widget_grab_default (button);
+      gtk_widget_show (button);
+  }
+  if (!GTK_WIDGET_VISIBLE (window))
+     gtk_widget_show (window);
+  else
+     gtk_widget_destroy (window);
+
+}
+
 void protoconfig()
 {
    GtkWidget *scrolled_win, *separator;
@@ -1085,16 +1294,14 @@ void protoconfig()
    if (!protoconfigwindow)
    {
       protoconfigwindow= gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_widget_set_name(protoconfigwindow, "Protocol configuration");
-      gtk_widget_set_usize(protoconfigwindow, 260,400);
+      gtk_widget_set_name(protoconfigwindow, "Protoconfig");
+      gtk_widget_set_usize(protoconfigwindow, PROTO_C_WIDTH, PROTO_C_HEIGHT);
       
-      gtk_widget_set_uposition(protoconfigwindow, 1280-572-5, 0);
-
       gtk_window_set_policy (GTK_WINDOW(protoconfigwindow), TRUE, TRUE, FALSE);
          
       gtk_signal_connect (GTK_OBJECT (protoconfigwindow), "destroy",
                           GTK_SIGNAL_FUNC(gtk_widget_destroyed), &protoconfigwindow);
-      gtk_window_set_title (GTK_WINDOW (protoconfigwindow), "Protocols");
+      gtk_window_set_title (GTK_WINDOW (protoconfigwindow), "Configure Protocols");
       gtk_container_border_width (GTK_CONTAINER (protoconfigwindow), 0);
       box1 = gtk_vbox_new (FALSE, 0);
       gtk_container_add (GTK_CONTAINER (protoconfigwindow), box1);
@@ -1219,17 +1426,28 @@ void savelist()
    if (!GTK_WIDGET_VISIBLE(window))
       gtk_widget_show(window);
 }
-
+#ifdef GTK_HAVE_FEATURES_1_1_0
+static GtkWidget *create_filemenu(GtkAccelGroup *accel)
+#else
 static GtkWidget *create_filemenu(GtkAcceleratorTable *accel)
+#endif
 {
    GtkWidget *menu;
    GtkWidget *menuitem;
    
    menu = gtk_menu_new();
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_menu_set_accel_group(GTK_MENU(menu), accel);
+#else
    gtk_menu_set_accelerator_table(GTK_MENU(menu), accel);
+#endif   
    menuitem = gtk_menu_item_new_with_label("Open list   Ctl-O");
    gtk_menu_append(GTK_MENU(menu), menuitem);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(menuitem, "activate", accel, 'O', GDK_CONTROL_MASK, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(menuitem), "activate", 'O', GDK_CONTROL_MASK);
+#endif
    gtk_widget_show(menuitem);
    gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
                              GTK_SIGNAL_FUNC(openlist), NULL);
@@ -1242,7 +1460,11 @@ static GtkWidget *create_filemenu(GtkAcceleratorTable *accel)
 
    menuitem = gtk_menu_item_new();
    gtk_menu_append(GTK_MENU(menu), menuitem);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(menuitem, "activate", accel, 'S', GDK_CONTROL_MASK, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(menuitem), "activate", 'S', GDK_CONTROL_MASK);
+#endif
    gtk_widget_show(menuitem);
    
    menuitem = gtk_menu_item_new_with_label("Clear list");
@@ -1263,32 +1485,56 @@ static GtkWidget *create_filemenu(GtkAcceleratorTable *accel)
    return menu;
 }
 
+#ifdef GTK_HAVE_FEATURES_1_1_0
+static GtkWidget *create_configmenu(GtkAccelGroup *accel)
+#else
 static GtkWidget *create_configmenu(GtkAcceleratorTable *accel)
+#endif
 {
    GtkWidget *menu;
    GtkWidget *menuitem;
    
    menu = gtk_menu_new();
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_menu_set_accel_group(GTK_MENU(menu), accel);
+#else
    gtk_menu_set_accelerator_table(GTK_MENU(menu), accel);
-
+#endif
    menuitem = gtk_menu_item_new_with_label("Configure protocols");
    gtk_menu_append(GTK_MENU(menu), menuitem);
-   gtk_accelerator_table_install(accel, GTK_OBJECT(menuitem), "activate", 'O', GDK_CONTROL_MASK);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(menuitem, "activate", accel, 'N', GDK_CONTROL_MASK);
+#else
+   gtk_accelerator_table_install(accel, GTK_OBJECT(menuitem), "activate", 'N', GDK_CONTROL_MASK);
+#endif
    gtk_widget_show(menuitem);
    gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
                              GTK_SIGNAL_FUNC(protoconfig), NULL);
 
+   menuitem = gtk_menu_item_new_with_label("Select interface");
+   gtk_menu_append(GTK_MENU(menu), menuitem);
+   gtk_widget_show(menuitem);
+   gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
+                             GTK_SIGNAL_FUNC(interfacesel), NULL);
+
 
    return menu;
 }
-
+#ifdef GTK_HAVE_FEATURES_1_1_0
+static GtkWidget *create_helpmenu(GtkAccelGroup *accel)
+#else
 static GtkWidget *create_helpmenu(GtkAcceleratorTable *accel)
+#endif
 {
    GtkWidget *menu;
    GtkWidget *menuitem;
    
    menu = gtk_menu_new();
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_
+#else
    gtk_menu_set_accelerator_table(GTK_MENU(menu), accel);
+#endif
    menuitem = gtk_menu_item_new_with_label("About");
    gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
                              GTK_SIGNAL_FUNC(switchabout), NULL);
@@ -1329,16 +1575,20 @@ void create_main_window()
                       GTK_SIGNAL_FUNC(gtk_false),
                       NULL);
    
-   gtk_widget_set_usize(mainwindow, 900, 580);
+   gtk_widget_set_usize(mainwindow, MAIN_WIDTH, MAIN_HEIGHT);
    gtk_widget_realize(mainwindow);
    gtk_widget_set_uposition(mainwindow,0,0);
    sprintf(strbuf, "karpski %s", KARPSKI_VERSION);
    gtk_window_set_title(GTK_WINDOW(mainwindow), strbuf);
    gtk_container_border_width(GTK_CONTAINER(mainwindow), 0);
-   
+
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   accel = gtk_accel_group_new();
+   gtk_window_add_accel_group(GTK_WINDOW(mainwindow), accel);
+#else   
    accel = gtk_accelerator_table_new();
    gtk_window_add_accelerator_table(GTK_WINDOW(mainwindow), accel);
-
+#endif
    box1 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(mainwindow), box1);
    gtk_widget_show(box1);
@@ -1408,7 +1658,11 @@ void create_main_window()
                       (GtkSignalFunc) start_capturing,
                       NULL);
    gtk_widget_show(button);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'S', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'S', 0);
+#endif
    gtk_tooltips_set_tip(tooltips, button, "Start the scanning process (shortcut=S)", "ContextHelp/buttons/start");
    
    button = gtk_button_new_with_label("Stop");
@@ -1418,8 +1672,11 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "Stop the scanner from scanning (shortcut=O)", "ContextHelp/buttons/stop");
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'O', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'O', 0);
-
+#endif
    separator = gtk_vseparator_new();
    gtk_box_pack_start(GTK_BOX(box2), separator, FALSE, TRUE, 0);
    gtk_widget_show(separator);
@@ -1431,8 +1688,11 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "View/Hide the log window (shortcut=L)", "ContextHelp/buttons/log");
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'L', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'L', 0);
-
+#endif
    button = gtk_button_new_with_label("Overall Stats");
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
@@ -1440,7 +1700,12 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "Look at the overall K.ARP.SKI statistics (shortcut=V)", "ContextHelp/buttons/stats");
+
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'V', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'V', 0);
+#endif
 
    separator = gtk_vseparator_new();
    gtk_box_pack_start(GTK_BOX(box2), separator, FALSE, TRUE, 0);
@@ -1453,7 +1718,11 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "View the connections associated with this MAC address (shortcut=C)", "ContextHelp/buttons/connections");
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'C', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'C', 0);
+#endif
 
    button = gtk_button_new_with_label("All connections");
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
@@ -1462,8 +1731,12 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "View a list of all connections (shortcut=A)", "ContextHelp/buttons/allconnections");
-   gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'A', 0);
 
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'A', 0, 0);
+#else
+   gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'A', 0);
+#endif
    button = gtk_button_new_with_label("Launch");
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
@@ -1475,7 +1748,12 @@ void create_main_window()
    }
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "Launch a program vs. the selected MAC address (shortcut=L) ", "ContextHelp/buttons/smashy");
+
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'L', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'L', 0);
+#endif
    
    button = gtk_button_new_with_label("Watch");
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
@@ -1484,8 +1762,12 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "Start watching all traffic to/from the selected MAC address (shortcut=W)", "ContextHelp/buttons/watchall");
-   gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'W', 0);
 
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'W', 0, 0);
+#else
+   gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'W', 0);
+#endif
    
    button = gtk_button_new_with_label("Protocols");
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
@@ -1494,8 +1776,12 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "View/Hide a semi-realtime list of the protocols used (shortcut=P)", "ContextHelp/buttons/protocols");
-   gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'P', 0);
 
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'P', 0, 0);
+#else
+   gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'P', 0);
+#endif
    
    button = gtk_button_new_with_label("Info");
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
@@ -1504,7 +1790,12 @@ void create_main_window()
                       NULL);
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "View info on the selected MAC address (shortcut=I)", "ContextHelp/buttons/info");
+
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'I', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked", 'I', 0);
+#endif
 
    button = gtk_button_new_with_label("Quit");
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
@@ -1515,9 +1806,12 @@ void create_main_window()
    gtk_widget_show(button);
    gtk_tooltips_set_tip(tooltips, button, "Quit without saving anything (shortcut=Ctrl-Q)", "ContextHelp/buttons/quit");
    gtk_widget_show(mainwindow);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+   gtk_widget_add_accelerator(button, "clicked", accel, 'Q', 0, 0);
+#else
    gtk_accelerator_table_install(accel, GTK_OBJECT(button), "clicked",
    				 'Q', GDK_CONTROL_MASK);
-   
+#endif   
    return;
 
 }
